from starfishX.markminervini.trendtemplate import trendMA_YMF
from starfishX.markminervini.trendtemplate import trendMA_Year
from starfishX.markminervini.trendtemplate import check52WeekHighLow
from starfishX.markminervini.trendtemplate import ROC_Basket

__version__ = "markminervini 0.1" #2022-oct-31